"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const __1 = require("..");
exports.USER_SERVICE = 'UserService';
exports.normaliseEmail = (email) => email.toLowerCase();
class AbstractUserService {
    constructor(loginIdentifierRepository) {
        this.loginIdentifierRepository = loginIdentifierRepository;
        this.baseLogger = __1.createLogger('abstract-user-service');
    }
    async getByEmail(context, email) {
        const loginIdentifier = await this.loginIdentifierRepository.get(context, exports.normaliseEmail(email));
        return loginIdentifier && this.get(context, loginIdentifier.userId);
    }
    async createOrUpdate(context, updates, beforeUpdate = () => { }) {
        const existingUser = await this.getByEmail(context, updates.email);
        if (existingUser) {
            beforeUpdate(existingUser);
        }
        return existingUser ? this.updateRetrievedUser(context, existingUser, updates) : this.create(context, updates);
    }
    async create(context, user) {
        const normalisedEmail = exports.normaliseEmail(user.email);
        await this.validateEmailAddressAvailable(context, normalisedEmail);
        const createdUser = await this.createUser(context, Object.assign({}, user, { email: normalisedEmail, roles: user.roles || [] }));
        await this.createLoginIdentifier(context, normalisedEmail, createdUser.id);
        this.baseLogger.info(`Created new user ${user.email}`);
        return createdUser;
    }
    async update(context, id, updates) {
        const user = await this.get(context, id);
        if (!user) {
            throw new Error(`No user exists with id: ${id}`);
        }
        return await this.updateRetrievedUser(context, user, updates);
    }
    async updateRetrievedUser(context, user, updates) {
        if (updates.roles && updates.roles.includes('super')) {
            throw new Error('Cannot assign super role to users');
        }
        const normalisedEmail = updates.email && exports.normaliseEmail(updates.email);
        if (normalisedEmail && normalisedEmail !== user.email) {
            this.baseLogger.info(`Email changed from [${user.email}] to [${normalisedEmail}]. Changing email for user id [${user.id}]`);
            await this.validateEmailAddressAvailable(context, normalisedEmail);
            await Promise.all([
                this.loginIdentifierRepository.delete(context, user.email),
                this.createLoginIdentifier(context, normalisedEmail, user.id),
            ]);
        }
        const userUpdates = (normalisedEmail && Object.assign({}, updates, { email: normalisedEmail })) || updates;
        return this.updateUser(context, user, userUpdates);
    }
    async createLoginIdentifier(context, email, userId) {
        return this.loginIdentifierRepository.save(context, {
            id: email,
            createdAt: new Date(),
            userId,
        });
    }
    async validateEmailAddressAvailable(context, email) {
        const existing = await this.loginIdentifierRepository.get(context, email);
        if (existing) {
            throw new Error(`Email address already exists: ${email}`);
        }
    }
}
tslib_1.__decorate([
    __1.Transactional(),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object, Object, Function]),
    tslib_1.__metadata("design:returntype", Promise)
], AbstractUserService.prototype, "createOrUpdate", null);
tslib_1.__decorate([
    __1.Transactional(),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object, Object]),
    tslib_1.__metadata("design:returntype", Promise)
], AbstractUserService.prototype, "create", null);
tslib_1.__decorate([
    __1.Transactional(),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object, String, Object]),
    tslib_1.__metadata("design:returntype", Promise)
], AbstractUserService.prototype, "update", null);
exports.AbstractUserService = AbstractUserService;
//# sourceMappingURL=user.service.js.map