"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const DatastoreStore = require("@google-cloud/connect-datastore");
const datastore_1 = require("@google-cloud/datastore");
const express = require("express");
const session = require("express-session");
const csp = require("helmet-csp");
const passport = require("passport");
const csrf_interceptor_1 = require("./auth/csrf.interceptor");
const logging_1 = require("./gcloud/logging");
const types_1 = require("./util/types");
const minutesToMilliseconds = (minutes) => minutes * 60 * 1000;
const MAX_AGE_DEFAULT = minutesToMilliseconds(2 * 60);
exports.configureExpress = (expressApp, options) => {
    const SessionStore = DatastoreStore(session);
    expressApp.use(csp(options.csp || {
        directives: {
            defaultSrc: ["'none'"],
            scriptSrc: ["'self'"],
            styleSrc: ["'self'", "'unsafe-inline'", 'https://fonts.googleapis.com'],
            fontSrc: ["'self'", 'https://fonts.gstatic.com'],
            imgSrc: ["'self'", 'data: ', 'https://secure.gravatar.com/'],
            connectSrc: ["'self'", 'https://www.googleapis.com'],
            manifestSrc: ["'self'"],
        },
    }));
    const { staticAssets } = options;
    if (staticAssets) {
        const staticOptions = staticAssets.options || {
            index: false,
        };
        if (staticAssets.prefix) {
            logging_1.rootLogger.info(`Serving static assets from ${staticAssets.root} on prefix ${staticAssets.prefix}`);
            expressApp.use(staticAssets.prefix, express.static(staticAssets.root, staticOptions));
        }
        else {
            logging_1.rootLogger.info(`Serving static assets from ${staticAssets.root}`);
            expressApp.use(express.static(staticAssets.root, staticOptions));
        }
    }
    let secure = (options.session.cookie && options.session.cookie.secure) || false;
    if (process.env.NODE_ENV === 'production' && process.env.APP_ENGINE_ENVIRONMENT) {
        expressApp.set('trust proxy', true);
        secure = true;
        logging_1.rootLogger.info('Cookie secured for prod');
    }
    const sessionAge = options.sessionTimeoutInMinutes
        ? minutesToMilliseconds(options.sessionTimeoutInMinutes)
        : MAX_AGE_DEFAULT;
    logging_1.rootLogger.info(`Session age set to: ${sessionAge} ms`);
    expressApp.use(session({
        saveUninitialized: true,
        resave: true,
        rolling: true,
        store: new SessionStore({
            dataset: new datastore_1.Datastore({
                prefix: 'express-sessions',
                apiEndpoint: options.session.apiEndpoint,
                projectId: options.session.projectId,
            }),
        }),
        secret: options.session.secret,
        cookie: Object.assign({ maxAge: sessionAge }, options.session.cookie, { secure }),
    }));
    const { ignorePaths = [/^\/(tasks\/|system\/).*/], sameSite = true, disabled: csrfDisabled = false } = options.csrf || {};
    const unless = (exclusions, middleware) => {
        return (req, res, next) => {
            const matchesExclusion = types_1.asArray(exclusions).some(path => typeof path.test === 'function' ? path.test(req.path) : path === req.path);
            matchesExclusion ? next() : middleware(req, res, next);
        };
    };
    expressApp.use(passport.initialize());
    expressApp.use(passport.session());
    if (!csrfDisabled) {
        const csrfValidator = csrf_interceptor_1.CsrfValidatorWithOptions({ sameSite });
        expressApp.use(unless(ignorePaths, csrfValidator));
    }
};
//# sourceMappingURL=configure.js.map