"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const logging_1 = require("../gcloud/logging");
const guards_1 = require("../util/guards");
const mail_logging_sender_1 = require("./mail-logging.sender");
class MailWhitelistSender {
    constructor(mailSender, configurationProvider) {
        this.mailSender = mailSender;
        this.configurationProvider = configurationProvider;
        this.logger = logging_1.createLogger('mail-whitelist-sender');
        this.filterAddresses = (actualAddresses) => {
            if (guards_1.isEmpty(actualAddresses)) {
                return [];
            }
            if (typeof actualAddresses === 'string') {
                const addresses = actualAddresses.split(',');
                return addresses
                    .map(address => address.trim())
                    .filter(address => this.filterAddress(address))
                    .join(',');
            }
            return lodash_1.castArray(actualAddresses).filter(address => this.filterAddress(address));
        };
        this.filterAddress = (address) => {
            const email = typeof address === 'string' ? address.trim() : address.address.trim();
            if (this.emailWhitelist.includes(email)) {
                return true;
            }
            this.logger.info(`Filtering non-whitelist email: ${email}`);
            return false;
        };
        const { devHooks } = this.configurationProvider;
        if (!devHooks || guards_1.isEmpty(devHooks.emailWhitelist)) {
            throw new Error('No whitelisted email address(es) defined');
        }
        this.logger.info('MailSender instance is MailWhitelistSender');
        this.logger.info(`Configuring mail whitelist: ${devHooks.emailWhitelist}`);
        this.emailWhitelist = devHooks.emailWhitelist;
    }
    async send(context, mailOptions) {
        const filteredAddresses = {
            to: this.filterAddresses(mailOptions.to),
            cc: this.filterAddresses(mailOptions.cc),
            bcc: this.filterAddresses(mailOptions.bcc),
        };
        if (guards_1.isEmpty(filteredAddresses.to) && guards_1.isEmpty(filteredAddresses.cc) && guards_1.isEmpty(filteredAddresses.bcc)) {
            this.logger.warn('Cannot send email - no whitelisted recipients found');
            return new mail_logging_sender_1.MailLoggingSender().send(context, mailOptions);
        }
        else {
            this.logger.info('Sending email using whitelisted addresses: ', filteredAddresses);
            return this.mailSender.send(context, Object.assign({}, mailOptions, filteredAddresses));
        }
    }
}
exports.MailWhitelistSender = MailWhitelistSender;
//# sourceMappingURL=mail-whitelist.sender.js.map