"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const repository_1 = require("../datastore/repository");
const types_1 = require("../util/types");
const __1 = require("..");
class SearchableRepository extends repository_1.Repository {
    constructor(datastore, searchService, kind, validator, options) {
        super(datastore, kind, validator, options);
        this.searchService = searchService;
        this.options = options;
        this.baseLogger = __1.createLogger('searchable-repository');
    }
    async save(context, entities) {
        const savedEntities = await super.save(context, entities);
        await this.index(savedEntities);
        return savedEntities;
    }
    async update(context, entities) {
        const savedEntities = await super.update(context, entities);
        await this.index(savedEntities);
        return savedEntities;
    }
    async insert(context, entities) {
        const savedEntities = await super.insert(context, entities);
        await this.index(savedEntities);
        return savedEntities;
    }
    async upsert(context, entities) {
        const savedEntities = await super.upsert(context, entities);
        await this.index(savedEntities);
        return savedEntities;
    }
    async delete(context, ...ids) {
        await super.delete(context, ...ids);
        await this.searchService.delete(this.kind, ...ids);
    }
    async deleteAll(context) {
        await super.deleteAll(context);
        await this.searchService.deleteAll(this.kind);
    }
    async search(context, searchFields, sort) {
        const queryResults = await this.searchService.query(this.kind, searchFields, sort);
        const requests = await this.fetchResults(context, queryResults.ids);
        return requests;
    }
    async searchWithPagination(context, searchFields, sort, page) {
        const queryResults = await this.searchService.query(this.kind, searchFields, sort, page);
        const requests = await this.fetchResults(context, queryResults.ids);
        return {
            resultCount: queryResults.resultCount,
            limit: queryResults.limit,
            offset: queryResults.offset,
            results: requests,
        };
    }
    index(entities) {
        const entitiesArr = types_1.asArray(entities);
        const entries = entitiesArr.map(entity => {
            const fields = Object.keys(this.options.searchIndex).reduce((obj, fieldName) => {
                obj[fieldName] = entity[fieldName];
                return obj;
            }, {});
            return {
                id: entity.id,
                fields,
            };
        });
        return this.searchService.index(this.kind, entries);
    }
    async fetchResults(context, ids) {
        const results = await this.get(context, ids);
        if (results && results.some(result => !result)) {
            this.baseLogger.warn('Search results contained at least one null value - search index likely out of sync with datastore');
            return results.filter(result => !!result);
        }
        return results;
    }
}
exports.SearchableRepository = SearchableRepository;
//# sourceMappingURL=searchable.repository.js.map